import { useEffect, useRef, useState } from 'react';
import { CSSTransition } from 'react-transition-group';
import { COLORS } from 'constant';
import {
  AppsCatalogAppArticlePresentation,
  AppsCatalogAppScreenshotPresentation,
  AppsCatalogAppVideoPresentation,
} from 'patchkit_api_foundation';
import styled from 'styled-components';

import { ArrowLeftIcon, ArrowRightIcon } from 'assets/icons';
import { ArticleTile } from 'components/article-tile';
import { Button } from 'components/common/button';
import { LightBox } from 'components/lightbox';
import { MediaTile } from 'components/media-tile';

const Container = styled.div`
  max-width: 100%;
  height: 17.5rem;
  position: relative;
`;

interface StyleProps {
  isScrolledLeft: boolean;
}

const ItemsContainer = styled.div<StyleProps>`
  width: 100%;
  height: 100%;
  display: flex;
  align-items: center;
  overflow-x: scroll;
  overflow-y: hidden;
  scrollbar-width: none;
  scroll-behavior: smooth;
  -ms-overflow-style: none;
  &::-webkit-scrollbar {
    display: none;
  }

  -webkit-mask-image: linear-gradient(
    to right,
    ${(props) =>
      props.isScrolledLeft
        ? 'rgba(0,0,0,0), rgba(0,0,0,1) 10%, rgba(0,0,0,1) 90%, rgba(0,0,0,0)'
        : 'rgba(0,0,0,1) 90%, rgba(0,0,0,0)'}
  );
  -webkit-mask-size: 100% 100%;
  -webkit-mask-repeat: no-repeat;
  -webkit-mask-position: right;
`;

// FIXME: use grid or something, i don't want to
const Row = styled.div`
  width: 100%;
  height: 17.5rem;
  display: flex;
  flex-direction: column;
  flex-wrap: wrap;
  justify-content: space-between;
  align-content: flex-start;

  & > * {
    margin-left: 0.5rem;
  }
`;

const ButtonContainer = styled.div`
  position: absolute;
  top: 50%;
  transform: translateY(-50%);

  svg {
    width: 2rem;
    height: 2rem;
  }

  svg > path {
    stroke: ${COLORS.WHITE};
  }
`;

const LeftButtonContainer = styled(ButtonContainer)`
  left: 0;
  z-index: 1;
`;

const RightButtonContainer = styled(ButtonContainer)`
  right: 0;
  z-index: 1;
`;

// TODO: use types from foundation
interface Props {
  images: [string, AppsCatalogAppScreenshotPresentation][];
  videos: [string, AppsCatalogAppVideoPresentation][];
  articles: [string, AppsCatalogAppArticlePresentation][];
}

export const AppGallery = ({ videos, images, articles }: Props) => {
  const ref = useRef<HTMLDivElement>(null);
  const [choosedImgUrl, setChoosedImgUrl] = useState<string>();
  const [choosedVideoUrl, setChoosedVideoUrl] = useState<string>();

  const renderFirstElement = () => {
    if (videos.length > 0) {
      const ytId = videos[0][1].videoUrl.split('v=')[1];
      const url = `https://img.youtube.com/vi/${ytId}/0.jpg`;

      return (
        <MediaTile
          backgroundImg={url}
          big
          isVideo
          onClick={() => setChoosedVideoUrl(videos[0][1].videoUrl)}
        />
      );
    }

    if (images.length > 0) {
      return (
        <MediaTile
          backgroundImg={images[0][1].screenshotUrl}
          big
          onClick={() => setChoosedImgUrl(images[0][1].screenshotUrl)}
        />
      );
    }

    return <></>;
  };

  const renderRowVideos = () => {
    return videos.slice(1).map(([id, video]) => {
      const ytId = video.videoUrl.split('v=')[1];
      const url = `https://img.youtube.com/vi/${ytId}/0.jpg`;

      return (
        <MediaTile
          key={id}
          backgroundImg={url}
          isVideo
          onClick={() => setChoosedVideoUrl(video.videoUrl)}
        />
      );
    });
  };

  const renderRowImages = () => {
    const otherImages = videos.length === 0 ? images.slice(1) : images;

    return otherImages.map(([id, image]) => {
      return (
        <MediaTile
          key={id}
          backgroundImg={image.screenshotUrl}
          onClick={() => setChoosedImgUrl(image.screenshotUrl)}
        />
      );
    });
  };

  const renderRowArticles = () => {
    return articles
      .filter((entry) => entry[1].title)
      .map(([id, article]) => {
        return (
          <ArticleTile
            key={id}
            imageUrl={article.bannerUrl}
            title={article.title!}
            markdownBody={article.body.markdownBody}
          />
        );
      });
  };

  const scrollLeft = () => {
    if (ref.current) {
      ref.current.scrollLeft -= 360;
    }
  };

  const scrollRight = () => {
    if (ref.current) {
      ref.current.scrollLeft += 360;
    }
  };

  const [canScrollLeft, setCanScrollLeft] = useState(false);

  useEffect(() => {
    const interval = setInterval(() => {
      if (ref.current) {
        setCanScrollLeft(ref.current.scrollLeft > 0);
      }
    }, 200);

    return () => clearInterval(interval);
  }, []);

  return (
    <Container>
      {canScrollLeft && (
        <LeftButtonContainer>
          <Button onClick={scrollLeft}>
            <ArrowLeftIcon />
          </Button>
        </LeftButtonContainer>
      )}
      <ItemsContainer ref={ref} isScrolledLeft={canScrollLeft}>
        <CSSTransition classNames="fade" timeout={300} in={images.length > 0 || videos.length > 0}>
          {renderFirstElement()}
        </CSSTransition>

        <CSSTransition
          classNames="fade"
          timeout={300}
          in={images.length > 0 || videos.length > 0 || articles.length > 0}
        >
          <Row>
            {renderRowVideos()}
            {renderRowImages()}
            {renderRowArticles()}
          </Row>
        </CSSTransition>
      </ItemsContainer>
      <RightButtonContainer>
        {(videos.length > 0 || images.length > 0 || articles.length > 0) && (
          <Button onClick={scrollRight}>
            <ArrowRightIcon />
          </Button>
        )}
      </RightButtonContainer>
      <CSSTransition
        classNames="fade"
        timeout={300}
        in={choosedImgUrl !== undefined || choosedVideoUrl !== undefined}
        mountOnEnter
        unmountOnExit
      >
        <>
          {
            <LightBox
              videoUrl={choosedVideoUrl}
              imageUrl={choosedImgUrl}
              onClose={() => {
                setChoosedImgUrl(undefined);
                setChoosedVideoUrl(undefined);
              }}
            />
          }
        </>
      </CSSTransition>
    </Container>
  );
};
