import { useContext, useMemo } from 'react';
import {
  DownloadQueueContext,
  useAddInstallToQueue,
  useAddRepairToQueue,
  useAddUpdateToQueue,
  useCancelOngoingInstallTask,
  useCancelOngoingRepairTask,
  useCancelOngoingUpdateTask,
  useDoesAppNeedsRepair,
  useIsAppBeingInstalled,
  useIsAppBeingMoved,
  useIsAppBeingRepaired,
  useIsAppBeingUninstalled,
  useIsAppBeingUpdated,
  useIsAppInstalled,
  useIsAppNewVersionAvailable,
  useIsAppStarted,
  useIsInstallInQueue,
  useIsRepairInQueue,
  useIsUpdateInQueue,
  useLastInstallTaskState,
  useLastRepairTaskState,
  useLastUpdateTaskState,
  useLaunchApp,
  useOngoingInstallTaskState,
  useOngoingRepairTaskState,
  useOngoingUpdateTaskState,
  useWasAppInstallCancelled,
  useWasAppRepairCancelled,
  useWasAppUpdateCancelled,
} from 'patchkit-generic-patcher2_hooks';
import { AppBranchFilesOperationTaskProgress } from 'patchkit-patcher2_foundation';

import { ProgressBar } from 'components/common/progress-bar';
import { InstallAppButton } from 'components/install-app-button';
import { LaunchAppButton } from 'components/launch-app-button';
import { RepairAppButton } from 'components/repair-app-button';
import { UpdateAppButton } from 'components/update-app-button';

interface Props {
  appId: string;
  appBranchId: string;
}

export const AppStateComponent = ({ appId, appBranchId }: Props) => {
  const isInstalled = useIsAppInstalled({ appId, appBranchId });
  const doesNeedRepair = useDoesAppNeedsRepair({ appId, appBranchId });
  const isUpdateAvailable = useIsAppNewVersionAvailable({ appId, appBranchId });

  const isBeingUninstalled = useIsAppBeingUninstalled({ appId, appBranchId });
  const isBeingInstalled = useIsAppBeingInstalled({ appId, appBranchId });
  const isBeingUpdated = useIsAppBeingUpdated({ appId, appBranchId });
  const isBeingRepaired = useIsAppBeingRepaired({ appId, appBranchId });
  const isBeingMoved = useIsAppBeingMoved({ appId, appBranchId });
  const isStarted = useIsAppStarted({ appId, appBranchId });

  const wasLastInstallTaskCancelled = useWasAppInstallCancelled({ appId, appBranchId });
  const wasLastUpdateTaskCancelled = useWasAppUpdateCancelled({ appId, appBranchId });
  const wasLastRepairTaskCancelled = useWasAppRepairCancelled({ appId, appBranchId });

  const isInstallInQueue = useIsInstallInQueue({ appId, appBranchId });
  const isRepairInQueue = useIsRepairInQueue({ appId, appBranchId });
  const isUpdateInQueue = useIsUpdateInQueue({ appId, appBranchId });
  const launch = useLaunchApp({ appId, appBranchId });

  const addInstallTaskToQueue = useAddInstallToQueue({ appId, appBranchId });
  const addUpdateTaskToQueue = useAddUpdateToQueue({ appId, appBranchId });
  const addRepairTaskToQueue = useAddRepairToQueue({ appId, appBranchId });

  // INSTALL TASK
  if (isBeingInstalled) {
    return <OngoingInstallTaskProgressBar appId={appId} appBranchId={appBranchId} />;
  }

  if (wasLastInstallTaskCancelled) {
    return <PausedInstallTaskProgressBar appId={appId} appBranchId={appBranchId} />;
  }

  if (isInstallInQueue) {
    return <UndefinedTaskProgressBar title="Install queued" />;
  }

  // UPDATE TASK
  if (isBeingUpdated) {
    return <OngoingUpdateTaskProgressBar appId={appId} appBranchId={appBranchId} />;
  }

  if (wasLastUpdateTaskCancelled) {
    return <PausedUpdateTaskProgressBar appId={appId} appBranchId={appBranchId} />;
  }

  if (isUpdateInQueue) {
    return <UndefinedTaskProgressBar title="Update queued" />;
  }

  // REPAIR TASK
  if (isBeingRepaired) {
    return <OngoingRepairTaskProgressBar appId={appId} appBranchId={appBranchId} />;
  }

  if (wasLastRepairTaskCancelled) {
    return <PausedRepairTaskProgressBar appId={appId} appBranchId={appBranchId} />;
  }

  if (isRepairInQueue) {
    return <UndefinedTaskProgressBar title="Repair queued" />;
  }

  // UNINSTALL TASK
  if (isBeingUninstalled) {
    return <UndefinedTaskProgressBar title="Uninstalling" />;
  }

  // MOVE TASK
  if (isBeingMoved) {
    return <UndefinedTaskProgressBar title="Moving" />;
  }

  // BUTTONS
  if (isStarted) {
    return <LaunchAppButton onClick={() => ({})} />;
  }

  if (doesNeedRepair) {
    return <RepairAppButton onClick={addRepairTaskToQueue} />;
  }

  if (isUpdateAvailable) {
    return <UpdateAppButton onClick={addUpdateTaskToQueue} />;
  }

  if (isInstalled) {
    return <LaunchAppButton onClick={launch} />;
  }

  return <InstallAppButton onClick={addInstallTaskToQueue} />;
};

interface TaskProgressBarProps {
  title: string;
  taskState?: {
    progress?: AppBranchFilesOperationTaskProgress;
  };
  onPause?: () => void;
  onResume?: () => void;
}

const TaskProgressBar = ({ title, taskState, onPause, onResume }: TaskProgressBarProps) => {
  const progressPercentage = useMemo(() => {
    if (!taskState) {
      return undefined;
    }

    const { progress } = taskState;

    if (!progress) {
      return undefined;
    }

    return (progress.packagesProcessedBytesCount / progress.packagesTotalBytesCount) * 100;
  }, [taskState]);

  return (
    <ProgressBar
      progressPercentage={progressPercentage}
      taskName={title}
      onPaused={onPause}
      onResumed={onResume}
      isPaused={onResume !== undefined}
    />
  );
};

const UndefinedTaskProgressBar = ({ title }: { title: string }) => {
  return <TaskProgressBar title={title} />;
};

const OngoingInstallTaskProgressBar = ({
  appId,
  appBranchId,
}: {
  appId: string;
  appBranchId: string;
}) => {
  const taskState = useOngoingInstallTaskState({ appId, appBranchId });
  const pauseInstall = useCancelOngoingInstallTask({ appId, appBranchId });

  return <TaskProgressBar title="Installing" taskState={taskState} onPause={pauseInstall} />;
};

const PausedInstallTaskProgressBar = ({
  appId,
  appBranchId,
}: {
  appId: string;
  appBranchId: string;
}) => {
  const taskState = useLastInstallTaskState({ appId, appBranchId });
  const { resumeCurrentTask } = useContext(DownloadQueueContext);

  return <TaskProgressBar title="Install paused" taskState={taskState} onResume={resumeCurrentTask} />;
};

const OngoingUpdateTaskProgressBar = ({
  appId,
  appBranchId,
}: {
  appId: string;
  appBranchId: string;
}) => {
  const taskState = useOngoingUpdateTaskState({ appId, appBranchId });
  const pauseUpdate = useCancelOngoingUpdateTask({ appId, appBranchId });

  return <TaskProgressBar title="Updating" taskState={taskState} onPause={pauseUpdate} />;
};

const PausedUpdateTaskProgressBar = ({
  appId,
  appBranchId,
}: {
  appId: string;
  appBranchId: string;
}) => {
  const taskState = useLastUpdateTaskState({ appId, appBranchId });
  const { resumeCurrentTask } = useContext(DownloadQueueContext);

  return <TaskProgressBar title="Update paused" taskState={taskState} onResume={resumeCurrentTask} />;
};

const OngoingRepairTaskProgressBar = ({
  appId,
  appBranchId,
}: {
  appId: string;
  appBranchId: string;
}) => {
  const taskState = useOngoingRepairTaskState({ appId, appBranchId });
  const pauseRepair = useCancelOngoingRepairTask({ appId, appBranchId });

  return <TaskProgressBar title="Repairing" taskState={taskState} onPause={pauseRepair} />;
};

const PausedRepairTaskProgressBar = ({
  appId,
  appBranchId,
}: {
  appId: string;
  appBranchId: string;
}) => {
  const taskState = useLastRepairTaskState({ appId, appBranchId });
  const { resumeCurrentTask } = useContext(DownloadQueueContext);

  return <TaskProgressBar title="Repair paused" taskState={taskState} onResume={resumeCurrentTask} />;
};
