import { useCallback } from 'react';
import Sketch from 'react-p5';
import p5Types from 'p5';

const color = [102, 222, 114];
const step = 20;
const limit = 44;
const font = 'Manrope';

const drawGradient = (p5: p5Types) => {
  for (let i = 0; i < p5.height; i++) {
    p5.line(0, i, p5.width, i);
    const opacity = p5.map(i, 0, p5.height, 100, 0);
    p5.stroke([...color, opacity]);
  }
};

const drawClippingMask = (p5: p5Types, data: number[]) => {
  p5.beginShape();
  for (let i = 0; i < data.length; i++) {
    p5.vertex(step * i, p5.height - data[i]);
  }
  p5.vertex(step * (data.length - 1), p5.height);
  p5.vertex(p5.width, p5.height);
  p5.vertex(p5.width, 0);
  p5.vertex(0, 0);
  p5.endShape(p5.CLOSE);
};

const drawChartStroke = (p5: p5Types, data: number[]) => {
  p5.stroke(color);
  p5.strokeWeight(2);
  p5.noFill();
  p5.beginShape();
  for (let i = 0; i < data.length; i++) {
    const x = step * i;
    const y = p5.height - data[i];
    p5.vertex(x, y);
  }
  p5.endShape();
};

const drawVerticalLine = (p5: p5Types, data: number[]) => {
  for (let i = 0; i < 3; i++) {
    const opacity = p5.map(i, 0, 3, 100, 0);
    p5.stroke([...color, opacity]);
    p5.strokeWeight(1);
    p5.line(step * (data.length - 1) - i, 0, step * (data.length - 1) - i, p5.height);
  }
  for (let i = 0; i < 3; i++) {
    const opacity = p5.map(i, 0, 3, 100, 0);
    p5.stroke([...color, opacity]);
    p5.strokeWeight(1);
    p5.line(step * (data.length - 1) + i, 0, step * (data.length - 1) + i, p5.height);
  }

  p5.stroke(color);
  p5.strokeWeight(2);
  p5.line(step * (data.length - 1), 0, step * (data.length - 1), p5.height);
};

const drawText = (p5: p5Types, data: number[]) => {
  p5.textFont(font);
  p5.textSize(16);
  p5.noStroke();
  p5.fill('#fff');
  p5.text(`${data[data.length - 1].toFixed(1)} KB/s`, step * (data.length - 1) + 9, 12);
};

const scaleValues = (p5: p5Types, data: number[]) => {
  const max = Math.max(...data);

  if (max > p5.height) {
    return data.map((value) => {
      const min = Math.min(...data);
      return p5.map(value, min, max, 0, p5.height - 1);
    });
  }

  return data;
};

const sliceValues = (data: number[]) => {
  return data.length > limit ? data.slice(data.length - limit) : data;
};

export const DownloadChart = ({ data }: { data: number[] }) => {
  const setup = useCallback((p5: p5Types, canvasParentRef: Element) => {
    p5.createCanvas(968, 60).parent(canvasParentRef);
  }, []);

  const draw = useCallback(
    (p5: p5Types) => {
      p5.clear();
      p5.stroke(0);
      p5.fill('transparent');
      p5.strokeWeight(1);

      drawGradient(p5);
      p5.erase();
      drawClippingMask(p5, sliceValues(scaleValues(p5, data)));
      p5.noErase();
      drawChartStroke(p5, sliceValues(scaleValues(p5, data)));
      drawVerticalLine(p5, sliceValues(scaleValues(p5, data)));
      drawText(p5, sliceValues(data));
    },
    [data],
  );

  return <Sketch setup={setup} draw={draw} />;
};
