import { useEffect, useState } from 'react';
import ReactMarkdown from 'react-markdown';
import { useNavigate, useParams } from 'react-router';
import { COLORS } from 'constant';
import { AppVersionPresentation } from 'patchkit_api_foundation';
import { useAppBranchVersionsPresentation } from 'patchkit-generic-patcher2_hooks';
import remarkGfm from 'remark-gfm';
import styled from 'styled-components';

import { AppPageOptionsContainer } from 'components/app-page-options-container';
import { VersionLabelButton } from 'components/version-label-button';

const Container = styled.div`
  display: flex;
  width: 100%;
  height: 100%;
`;

const LabelsContainer = styled.div`
  display: flex;
  flex-direction: column;
  overflow-y: scroll;
  width: 6rem;
  padding-right: 1rem;

  & > * {
    margin-bottom: 0.25rem;
  }
`;

const Content = styled.div`
  display: flex;
  flex-direction: column;
  width: 100%;
  height: 100%;
  padding: 0 1rem;
  overflow-y: scroll;
  overflow-x: hidden;
  max-height: 100%;
`;

const DateDisplay = styled.p`
  font-size: 1rem;
  font-weight: 600;
  margin: 0;
  color: ${COLORS.WHITE};
  margin-bottom: 1rem;
`;

const Title = styled.h1`
  font-size: 2rem;
  font-weight: 800;
  margin: 0;
  margin-bottom: 1.5rem;
  color: ${COLORS.WHITE};
`;

const Description = styled.div`
  font-size: 1rem;
  font-weight: normal;
  margin: 0;
  margin-bottom: 1rem;
  color: ${COLORS.WHITE};

  img {
    max-width: 400px;
  }

  a {
    color: ${COLORS.WHITE};
  }

  & > :first-child {
    margin-top: 0;
  }

  & table {
    border-collapse: collapse;
    width: 100%;
  }

  & td,
  th {
    border: 0.0625rem solid ${COLORS.GRAT_TEXT};
    text-align: left;
    padding: 0.25rem;
  }
`;

interface Props {
  appId: string;
  appBranchId: string;
}

const AppChangelogContent = ({ appId, appBranchId }: Props) => {
  const navigate = useNavigate();
  const baseUrl = `/app/${appId}/branch/${appBranchId}`;

  const appVersions = useAppBranchVersionsPresentation({ appId, appBranchId });

  const [currentChangelog, setCurrentChangelog] = useState<AppVersionPresentation>();

  console.log(currentChangelog?.publishDate);

  useEffect(() => {
    if (appVersions.length > 0 && !currentChangelog) {
      setCurrentChangelog([...appVersions].reverse()[0][1]);
    }
  }, [appVersions, currentChangelog]);

  return (
    <AppPageOptionsContainer title="Releases" onClose={() => navigate(baseUrl)}>
      <Container>
        <LabelsContainer>
          {[...appVersions].reverse().map(([_, changelog]) => (
            <VersionLabelButton
              key={changelog.label}
              text={changelog.label}
              isActive={currentChangelog?.label === changelog.label}
              onClick={() => setCurrentChangelog(changelog)}
            />
          ))}
        </LabelsContainer>
        <Content>
          <DateDisplay>
            {currentChangelog?.publishDate && currentChangelog?.publishDate.toLocaleDateString()}
          </DateDisplay>
          <Title>{currentChangelog?.label}</Title>
          <Description>
            <ReactMarkdown remarkPlugins={[remarkGfm]}>
              {currentChangelog?.changelog || ''}
            </ReactMarkdown>
          </Description>
        </Content>
      </Container>
    </AppPageOptionsContainer>
  );
};

export const AppChangelog = () => {
  const { appId, appBranchId } = useParams();

  if (!appId || !appBranchId) {
    return <></>;
  }

  return <AppChangelogContent appId={appId} appBranchId={appBranchId} />;
};
