import { useEffect, useMemo, useState } from 'react';
import { useNavigate, useParams } from 'react-router';
import { COLORS } from 'constant';
import {
  useIsAppBeingInstalled,
  useIsAppBeingRepaired,
  useIsAppBeingUpdated,
  useIsInstallInQueue,
  useIsRepairInQueue,
  useIsUpdateInQueue,
  useLastInstallTaskState,
  useLastRepairTaskState,
  useLastUpdateTaskState,
  useOngoingInstallTaskState,
  useOngoingRepairTaskState,
  useOngoingUpdateTaskState,
  useWasAppInstallCancelled,
  useWasAppRepairCancelled,
  useWasAppUpdateCancelled,
} from 'patchkit-generic-patcher2_hooks';
import { AppBranchFilesOperationTaskProgress } from 'patchkit-patcher2_foundation';
import styled from 'styled-components';

import { AppPageOptionsContainer } from 'components/app-page-options-container';
import { DownloadChart } from 'components/download-chart';

const Container = styled.div`
  display: flex;
  flex-direction: column;
  width: 100%;
  height: 100%;
  justify-content: flex-end;
  padding: 0 0 3rem 2rem;
`;

const DataRow = styled.div`
  display: flex;
  max-width: 50%;
  margin-bottom: 2rem;
`;

const DataColumn = styled.div`
  display: flex;
  flex-direction: column;
  align-items: flex-start;
  margin-right: 2rem;
`;

const DataTitle = styled.p`
  font-size: 1rem;
  font-weight: 600;
  margin: 0;
  margin-bottom: 1.5rem;
  color: ${COLORS.GRAT_TEXT};
`;

const DataInfo = styled(DataTitle)`
  color: ${COLORS.WHITE};
`;

const OtherInfo = styled.h1`
  font-size: 1.5rem;
  font-weight: 600;
  color: ${COLORS.GRAT_TEXT};
`;

const SpeedsChartText = styled.div`
  display: flex;
  justify-content: space-between;
  font-size: 1rem;
  font-weight: 600;
  color: ${COLORS.GRAT_TEXT};
  width: 90%;
  margin-bottom: 0.5rem;
`;

interface Props {
  appId: string;
  appBranchId: string;
}

export const AppDownloadsContent = ({ appId, appBranchId }: Props) => {
  const navigate = useNavigate();
  const baseUrl = `/app/${appId}/branch/${appBranchId}`;

  const isBeingInstalled = useIsAppBeingInstalled({ appId, appBranchId });
  const isBeingUpdated = useIsAppBeingUpdated({ appId, appBranchId });
  const isBeingRepaired = useIsAppBeingRepaired({ appId, appBranchId });

  const wasLastInstallTaskCancelled = useWasAppInstallCancelled({ appId, appBranchId });
  const wasLastUpdateTaskCancelled = useWasAppUpdateCancelled({ appId, appBranchId });
  const wasLastRepairTaskCancelled = useWasAppRepairCancelled({ appId, appBranchId });

  const isInstallInQueue = useIsInstallInQueue({ appId, appBranchId });
  const isRepairInQueue = useIsRepairInQueue({ appId, appBranchId });
  const isUpdateInQueue = useIsUpdateInQueue({ appId, appBranchId });

  const renderDataComponent = () => {
    if (isBeingInstalled) {
      return (
        <DataComponent
          appId={appId}
          appBranchId={appBranchId}
          useProgressHook={useOngoingInstallTaskState}
        />
      );
    }

    if (isBeingUpdated) {
      return (
        <DataComponent
          appId={appId}
          appBranchId={appBranchId}
          useProgressHook={useOngoingUpdateTaskState}
        />
      );
    }

    if (isBeingRepaired) {
      return (
        <DataComponent
          appId={appId}
          appBranchId={appBranchId}
          useProgressHook={useOngoingRepairTaskState}
        />
      );
    }

    if (wasLastInstallTaskCancelled) {
      return (
        <DataComponent
          appId={appId}
          appBranchId={appBranchId}
          useProgressHook={useLastInstallTaskState}
        />
      );
    }

    if (wasLastUpdateTaskCancelled) {
      return (
        <DataComponent
          appId={appId}
          appBranchId={appBranchId}
          useProgressHook={useLastUpdateTaskState}
        />
      );
    }

    if (wasLastRepairTaskCancelled) {
      return (
        <DataComponent
          appId={appId}
          appBranchId={appBranchId}
          useProgressHook={useLastRepairTaskState}
        />
      );
    }

    if (isInstallInQueue) {
      return <OtherInfo>Install task is in queue</OtherInfo>;
    }

    if (isRepairInQueue) {
      return <OtherInfo>Repair task is in queue</OtherInfo>;
    }

    if (isUpdateInQueue) {
      return <OtherInfo>Update task is in queue</OtherInfo>;
    }

    return <OtherInfo>No downloads in progress</OtherInfo>;
  };

  const renderDownloadChart = () => {
    if (isBeingInstalled) {
      return (
        <DownloadSpeedsChart
          appId={appId}
          appBranchId={appBranchId}
          useProgressHook={useOngoingInstallTaskState}
        />
      );
    }

    if (isBeingUpdated) {
      return (
        <DownloadSpeedsChart
          appId={appId}
          appBranchId={appBranchId}
          useProgressHook={useOngoingUpdateTaskState}
        />
      );
    }

    if (isBeingRepaired) {
      return (
        <DownloadSpeedsChart
          appId={appId}
          appBranchId={appBranchId}
          useProgressHook={useOngoingRepairTaskState}
        />
      );
    }

    if (wasLastInstallTaskCancelled || wasLastUpdateTaskCancelled || wasLastRepairTaskCancelled) {
      return <DownloadChart data={[0, 0, 0, 0, 0, 0, 0, 0, 0, 0]} />;
    }

    return <></>;
  };

  return (
    <AppPageOptionsContainer title="Downloading info" onClose={() => navigate(baseUrl)}>
      <Container>
        {renderDataComponent()}
        {renderDownloadChart()}
      </Container>
    </AppPageOptionsContainer>
  );
};

interface DataComponentProps extends Props {
  useProgressHook: ({ appId, appBranchId }: { appId: string; appBranchId: string }) =>
    | {
        progress?: AppBranchFilesOperationTaskProgress;
      }
    | undefined;
}

const DataComponent = ({ appId, appBranchId, useProgressHook }: DataComponentProps) => {
  const taskState = useProgressHook({ appId, appBranchId });

  const packageSize = useMemo(() => {
    if (!taskState?.progress) {
      return '- GB';
    }

    const totalMegabytesCount = taskState.progress.packagesTotalBytesCount / 1024 / 1024;
    const totalGigabytesCount = totalMegabytesCount / 1024;

    return totalGigabytesCount < 1
      ? `${totalMegabytesCount.toFixed(2)} MB`
      : `${totalGigabytesCount.toFixed(0)} GB`;
  }, [taskState]);

  const currentDownloadedSize = useMemo(() => {
    if (!taskState?.progress) {
      return '- GB';
    }

    const totalMegabytesCount = taskState.progress.packagesProcessedBytesCount / 1024 / 1024;
    const totalGigabytesCount = totalMegabytesCount / 1024;

    return totalGigabytesCount < 1
      ? `${totalMegabytesCount.toFixed(2)} MB`
      : `${totalGigabytesCount.toFixed(0)} GB`;
  }, [taskState]);

  return (
    <DataRow>
      <DataColumn>
        <DataTitle>Operation name</DataTitle>
        <DataTitle>Package size</DataTitle>
        <DataTitle>Current downloaded size</DataTitle>
      </DataColumn>
      <DataColumn>
        <DataInfo>Downloading package</DataInfo>
        <DataInfo>{packageSize}</DataInfo>
        <DataInfo>{currentDownloadedSize}</DataInfo>
      </DataColumn>
    </DataRow>
  );
};

const DownloadSpeedsChart = ({ appId, appBranchId, useProgressHook }: DataComponentProps) => {
  const taskState = useProgressHook({ appId, appBranchId });

  const processed =
    taskState?.progress?.packagesProcessedBytesCount &&
    Math.floor((taskState?.progress?.packagesProcessedBytesCount / 1024 / 1024) * 100) / 100;

  const total =
    taskState?.progress?.packagesTotalBytesCount &&
    Math.floor((taskState?.progress?.packagesTotalBytesCount / 1024 / 1024) * 100) / 100;

  const [downloadSpeeds, setDownloadSpeeds] = useState<number[]>([0]);

  useEffect(() => {
    const interval = setInterval(() => {
      setDownloadSpeeds((prevState) => {
        const newSpeed = taskState?.progress?.packagesProcessedBytesCountPerSecond;

        if (newSpeed === undefined) {
          return prevState;
        }

        return [...prevState, newSpeed / 1024];
      });
    }, 400);

    return () => clearInterval(interval);
    // eslint-disable-next-line react-hooks/exhaustive-deps
  }, [taskState?.progress?.packagesProcessedBytesCountPerSecond]);

  return (
    <div>
      <SpeedsChartText>
        <span>Download</span>
        <span>{processed && total ? `${processed} MB of ${total} MB` : ''}</span>
      </SpeedsChartText>
      <DownloadChart data={downloadSpeeds} />
    </div>
  );
};

export const AppDownloads = () => {
  const { appId, appBranchId } = useParams();

  if (!appId || !appBranchId) {
    return <></>;
  }

  return <AppDownloadsContent appId={appId} appBranchId={appBranchId} />;
};
