import { useMemo } from 'react';
import { useNavigate, useParams } from 'react-router';
import { COLORS } from 'constant';
import {
  AppsCatalogAppArticlePresentation,
  AppsCatalogAppScreenshotPresentation,
  AppsCatalogAppVideoPresentation,
} from 'patchkit_api_foundation';
import {
  useAppPresentation,
  useAppState,
  useIsAppInstalled,
} from 'patchkit-generic-patcher2_hooks';
import styled from 'styled-components';

import { DownloadIcon, SettingsIcon } from 'assets/icons';
import { AppGallery } from 'components/app-gallery';
import { AppPageButton } from 'components/app-page-button';
import { AppStateComponent } from 'components/app-state-component';

const Container = styled.div`
  display: flex;
  flex-direction: column;
  width: 100%;
  height: 100%;
  padding: 0;
  overflow: hidden;
`;

const LinksContainer = styled.div`
  display: flex;
  justify-content: flex-end;
  margin-bottom: 1.5rem;

  & > * {
    margin-left: 1rem;
  }
`;

const BottomColumn = styled.div`
  display: flex;
  flex-direction: column;
  align-items: flex-start;
  height: 100%;
  justify-content: flex-end;
`;

const Title = styled.h1`
  font-size: 3rem;
  font-weight: 800;
  margin: 0;
  margin-bottom: 1rem;
  color: ${COLORS.WHITE};
`;

const Description = styled.p`
  font-size: 1rem;
  font-weight: 600;
  margin: 0;
  margin-bottom: 1rem;
  color: ${COLORS.WHITE};
`;

interface Props {
  appId: string;
  appBranchId: string;
}

const AppContent = ({ appId, appBranchId }: Props) => {
  const navigate = useNavigate();
  const baseUrl = `/app/${appId}/branch/${appBranchId}`;
  const appState = useAppState({ appId });
  const appPresentation = useAppPresentation({ appId });
  const isInstalled = useIsAppInstalled({ appId, appBranchId });

  const images = useMemo(() => {
    if (!appPresentation) return [];

    return Array.from(Object.entries(appPresentation.screenshots)).flatMap(
      ([screenshotId, screenshot]) =>
        screenshot === undefined ? [] : [[screenshotId, screenshot]],
    ) as [string, AppsCatalogAppScreenshotPresentation][];
  }, [appPresentation]);

  const articles = useMemo(() => {
    if (!appPresentation) return [];

    return Array.from(Object.entries(appPresentation.articles)).flatMap(([articleId, article]) =>
      article === undefined ? [] : [[articleId, article]],
    ) as [string, AppsCatalogAppArticlePresentation][];
  }, [appPresentation]);

  const videos = useMemo(() => {
    if (!appPresentation) return [];

    return Array.from(Object.entries(appPresentation.videos)).flatMap(([videoId, video]) =>
      video === undefined ? [] : [[videoId, video]],
    ) as [string, AppsCatalogAppVideoPresentation][];
  }, [appPresentation]);

  return (
    <Container>
      <LinksContainer>
        <AppPageButton onClick={() => navigate(`${baseUrl}/changelog`)}>
          <p>Releases</p>
        </AppPageButton>
        <AppPageButton onClick={() => navigate(`${baseUrl}/downloads`)}>
          <DownloadIcon />
        </AppPageButton>
        {isInstalled && (
          <AppPageButton onClick={() => navigate(`${baseUrl}/settings`)} iconWithFill>
            <SettingsIcon />
          </AppPageButton>
        )}
      </LinksContainer>

      <AppGallery images={images} articles={articles} videos={videos} />

      <BottomColumn>
        <Title>{appState?.name}</Title>
        <Description>{appState?.longDescription}</Description>
        <AppStateComponent appId={appId} appBranchId={appBranchId} />
      </BottomColumn>
    </Container>
  );
};

export const AppPage = () => {
  const { appId, appBranchId } = useParams();

  if (!appId || !appBranchId) {
    return <></>;
  }

  return <AppContent appId={appId} appBranchId={appBranchId} />;
};
