import { useState } from 'react';
import { useNavigate, useParams } from 'react-router';
import { CSSTransition } from 'react-transition-group';
import { COLORS } from 'constant';
import {
  useAddRepairToQueue,
  useAppState,
  useIsAppBeingInstalled,
  useIsAppBeingMoved,
  useIsAppBeingRepaired,
  useIsAppBeingUninstalled,
  useIsAppBeingUpdated,
  useIsAppInstalled,
  useIsAppStarted,
  useIsInstallInQueue,
  useIsRepairInQueue,
  useIsUpdateInQueue,
  useMoveApp,
  useShowPathSelect,
  useUninstallApp,
  useWasAppInstallCancelled,
  useWasAppRepairCancelled,
  useWasAppUpdateCancelled,
} from 'patchkit-generic-patcher2_hooks';
import styled from 'styled-components';

import { RepairIcon, UninstallIcon } from 'assets/icons';
import { AppPageOptionsContainer } from 'components/app-page-options-container';
import { AppPathInput } from 'components/app-path-input';
import { AppSettingsButton } from 'components/app-settings-button';
import { UninstallConfirmationPopup } from 'components/uninstall-confirmation-popup';

const Container = styled.div`
  display: flex;
  flex-direction: column;
  width: 100%;
  height: 100%;
  justify-content: space-between;
  padding: 0 0 3rem 2rem;
`;

const Option = styled.div`
  display: flex;
  flex-direction: column;
  align-items: flex-start;
  max-width: 50%;
`;

const OptionTitle = styled.h2`
  font-size: 1.5rem;
  font-weight: 700;
  margin: 0;
  margin-bottom: 1rem;
  color: ${COLORS.WHITE};
`;

const OptionSubtitle = styled.p`
  font-size: 0.875rem;
  font-weight: 600;
  margin: 0;
  margin-bottom: 0.5rem;
  color: ${COLORS.GRAT_TEXT};
`;

interface Props {
  appId: string;
  appBranchId: string;
}

export const AppSettingsContent = ({ appId, appBranchId }: Props) => {
  const navigate = useNavigate();
  const baseUrl = `/app/${appId}/branch/${appBranchId}`;

  const appState = useAppState({ appId });
  const appBranchState = appState?.branches[appBranchId];

  const isInstalled = useIsAppInstalled({ appId, appBranchId });
  const isBeingUninstalled = useIsAppBeingUninstalled({ appId, appBranchId });
  const isBeingInstalled = useIsAppBeingInstalled({ appId, appBranchId });
  const isBeingUpdated = useIsAppBeingUpdated({ appId, appBranchId });
  const isBeingRepaired = useIsAppBeingRepaired({ appId, appBranchId });
  const isBeingMoved = useIsAppBeingMoved({ appId, appBranchId });
  const isStarted = useIsAppStarted({ appId, appBranchId });

  const wasLastInstallTaskCancelled = useWasAppInstallCancelled({ appId, appBranchId });
  const wasLastUpdateTaskCancelled = useWasAppUpdateCancelled({ appId, appBranchId });
  const wasLastRepairTaskCancelled = useWasAppRepairCancelled({ appId, appBranchId });

  const isInstallInQueue = useIsInstallInQueue({ appId, appBranchId });
  const isRepairInQueue = useIsRepairInQueue({ appId, appBranchId });
  const isUpdateInQueue = useIsUpdateInQueue({ appId, appBranchId });

  const addRepairTaskToQueue = useAddRepairToQueue({ appId, appBranchId });
  const uninstallApp = useUninstallApp({ appId, appBranchId });
  const moveApp = useMoveApp({ appId, appBranchId });
  const showAppPathWindow = useShowPathSelect({ appId, appBranchId });
  const [showUninstallConfirmation, setShowUninstallConfirmation] = useState(false);

  const canDoTask =
    isInstalled &&
    !isBeingInstalled &&
    !isBeingUpdated &&
    !isBeingRepaired &&
    !isBeingMoved &&
    !isBeingUninstalled &&
    !isStarted &&
    !isInstallInQueue &&
    !isRepairInQueue &&
    !isUpdateInQueue &&
    !wasLastInstallTaskCancelled &&
    !wasLastUpdateTaskCancelled &&
    !wasLastRepairTaskCancelled;

  return (
    <AppPageOptionsContainer title="Settings" onClose={() => navigate(baseUrl)}>
      <Container>
        <Option>
          <OptionTitle>Game files path</OptionTitle>
          <AppPathInput
            path={appBranchState?.dir?.path || ''}
            onClick={
              canDoTask
                ? async () => {
                    const path = await showAppPathWindow();
                    if (path) {
                      await moveApp(path);
                    }
                  }
                : () => ({})
            }
          />
        </Option>
        <Option>
          <OptionTitle>Uninstall game</OptionTitle>
          <OptionSubtitle>We delete all files assosiated with this game.</OptionSubtitle>
          <AppSettingsButton
            text="Uninstall"
            onClick={canDoTask ? () => setShowUninstallConfirmation(true) : () => ({})}
            icon={<UninstallIcon />}
          />
        </Option>
        <Option>
          <OptionTitle>Repair game</OptionTitle>
          <OptionSubtitle>
            Some problems with launching a game? Try to restore game files.
          </OptionSubtitle>
          <AppSettingsButton
            text="Repair"
            onClick={
              canDoTask
                ? () => {
                    addRepairTaskToQueue();
                    navigate(baseUrl);
                  }
                : () => ({})
            }
            icon={<RepairIcon />}
            iconWithFill
          />
        </Option>
      </Container>
      <CSSTransition
        classNames="fade"
        timeout={300}
        in={showUninstallConfirmation}
        mountOnEnter
        unmountOnExit
      >
        <UninstallConfirmationPopup
          onAccept={() => uninstallApp().then(() => navigate(baseUrl))}
          onClose={() => setShowUninstallConfirmation(false)}
        />
      </CSSTransition>
    </AppPageOptionsContainer>
  );
};

export const AppSettings = () => {
  const { appId, appBranchId } = useParams();

  if (!appId || !appBranchId) {
    return <></>;
  }

  return <AppSettingsContent appId={appId} appBranchId={appBranchId} />;
};
